@@ -1,5 +1,5 @@
 /* libcap-ng.c --
- * Copyright 2009-10, 2013, 2017, 2020 Red Hat Inc.
+ * Copyright 2009-10, 2013, 2017, 2020-21 Red Hat Inc.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -12,9 +12,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; see the file COPYING.LIB. If not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor
+ * Boston, MA 02110-1335, USA.
  *
  * Authors:
  *      Steve Grubb <sgrubb@redhat.com>
@@ -56,9 +57,22 @@ unsigned int last_cap hidden = 0;
  * 2.6.24 kernel	XATTR_NAME_CAPS
  * 2.6.25 kernel	PR_CAPBSET_DROP, CAPABILITY_VERSION_2
  * 2.6.26 kernel	PR_SET_SECUREBITS, SECURE_*_LOCKED, VERSION_3
+ * 3.5    kernel	PR_SET_NO_NEW_PRIVS
  * 4.3    kernel	PR_CAP_AMBIENT
  * 4.14   kernel	VFS_CAP_REVISION_3
  */
+#ifdef PR_CAPBSET_DROP
+static int HAVE_PR_CAPBSET_DROP = 0;
+#endif
+#ifdef PR_SET_SECUREBITS
+static int HAVE_PR_SET_SECUREBITS = 0;
+#endif
+#ifdef PR_SET_NO_NEW_PRIVS
+static int HAVE_PR_SET_NO_NEW_PRIVS = 0;
+#endif
+#ifdef PR_CAP_AMBIENT
+static int HAVE_PR_CAP_AMBIENT = 0;
+#endif
 
 /* External syscall prototypes */
 extern int capset(cap_user_header_t header, cap_user_data_t data);
@@ -201,6 +215,12 @@ static inline int test_cap(unsigned int cap)
 static void init_lib(void) __attribute__ ((constructor));
 static void init_lib(void)
 {
+       // This is so dynamic libraries don't re-init
+       static unsigned int run_once = 0;
+       if (run_once)
+               return;
+       run_once = 1;
+
 #ifdef HAVE_PTHREAD_H
 	pthread_atfork(NULL, NULL, deinit);
 #endif
@@ -246,10 +266,38 @@ fail:
 			}
 		}
 	}
+	// Detect prctl options at runtime
+#ifdef PR_CAPBSET_DROP
+	errno = 0;
+	prctl(PR_CAPBSET_READ, 0, 0, 0, 0);
+	if (!errno)
+		HAVE_PR_CAPBSET_DROP = 1;
+#endif
+#ifdef PR_SET_SECUREBITS
+	errno = 0;
+	prctl(PR_GET_SECUREBITS, 0, 0, 0, 0);
+	if (!errno)
+		HAVE_PR_SET_SECUREBITS = 1;
+#endif
+#ifdef PR_SET_NO_NEW_PRIVS
+	errno = 0;
+	prctl(PR_GET_NO_NEW_PRIVS, 0, 0, 0, 0);
+	if (!errno)
+		HAVE_PR_SET_NO_NEW_PRIVS = 1;
+#endif
+#ifdef PR_CAP_AMBIENT
+	errno = 0;
+	prctl(PR_CAP_AMBIENT, PR_CAP_AMBIENT_IS_SET, 0, 0, 0);
+	if (!errno)
+		HAVE_PR_CAP_AMBIENT = 1;
+#endif
 }
 
 static void init(void)
 {
+	// This is so static libs get initialized
+	init_lib();
+
 	if (m.state != CAPNG_NEW)
 		return;
 
@@ -291,12 +339,16 @@ void capng_clear(capng_select_t set)
 	if (set & CAPNG_SELECT_CAPS)
 		memset(&m.data, 0, sizeof(cap_data_t));
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 	if (set & CAPNG_SELECT_BOUNDS)
 		memset(m.bounds, 0, sizeof(m.bounds));
+}
 #endif
 #ifdef PR_CAP_AMBIENT
+if (HAVE_PR_CAP_AMBIENT) {
 	if (set & CAPNG_SELECT_AMBIENT)
 		memset(m.ambient, 0, sizeof(m.ambient));
+}
 #endif
 	m.state = CAPNG_INIT;
 }
@@ -323,18 +375,22 @@ void capng_fill(capng_select_t set)
 		}
 	}
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 	if (set & CAPNG_SELECT_BOUNDS) {
 		unsigned i;
 		for (i=0; i<sizeof(m.bounds)/sizeof(__u32); i++)
 			m.bounds[i] = 0xFFFFFFFFU;
 	}
+}
 #endif
 #ifdef PR_CAP_AMBIENT
+if (HAVE_PR_CAP_AMBIENT) {
 	if (set & CAPNG_SELECT_AMBIENT) {
 		unsigned i;
 		for (i=0; i<sizeof(m.ambient)/sizeof(__u32); i++)
 			m.ambient[i] = 0xFFFFFFFFU;
 	}
+}
 #endif
 	m.state = CAPNG_INIT;
 }
@@ -468,6 +524,9 @@ static int get_ambient_set(void)
 }
 #endif
 
+/*
+ * Returns 0 on success and -1 on failure
+ */
 int capng_get_caps_process(void)
 {
 	int rc;
@@ -481,14 +540,18 @@ int capng_get_caps_process(void)
 	if (rc == 0) {
 		m.state = CAPNG_INIT;
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 		rc = get_bounding_set();
 		if (rc < 0)
 			m.state = CAPNG_ERROR;
+}
 #endif
 #ifdef PR_CAP_AMBIENT
+if (HAVE_PR_CAP_AMBIENT) {
 		rc = get_ambient_set();
 		if (rc < 0)
 			m.state = CAPNG_ERROR;
+}
 #endif
 	}
 
@@ -578,6 +641,8 @@ int capng_get_caps_fd(int fd)
 	rc = load_data(&filedata, rc);
 	if (rc == 0)
 		m.state = CAPNG_INIT;
+	else
+		m.state = CAPNG_ERROR; // If load data failed, malformed data
 
 	return rc;
 #endif
@@ -622,10 +687,12 @@ static void update_bounding_set(capng_act_t action, unsigned int capability,
 	unsigned int idx)
 {
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 	if (action == CAPNG_ADD)
 		m.bounds[idx] |= MASK(capability);
 	else
 		m.bounds[idx] &= ~(MASK(capability));
+}
 #endif
 }
 
@@ -633,10 +700,12 @@ static void update_ambient_set(capng_act_t action, unsigned int capability,
 	unsigned int idx)
 {
 #ifdef PR_CAP_AMBIENT
+if (HAVE_PR_CAP_AMBIENT) {
 	if (action == CAPNG_ADD)
 		m.ambient[idx] |= MASK(capability);
 	else
 		m.ambient[idx] &= ~(MASK(capability));
+}
 #endif
 }
 
@@ -723,9 +792,11 @@ int capng_apply(capng_select_t set)
 
 	if (set & CAPNG_SELECT_BOUNDS) {
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 		struct cap_ng state;
 		memcpy(&state, &m, sizeof(state)); /* save state */
-		capng_get_caps_process();
+		if (capng_get_caps_process())
+			return -9;
 		if (capng_have_capability(CAPNG_EFFECTIVE, CAP_SETPCAP)) {
 			unsigned int i;
 			memcpy(&m, &state, sizeof(m)); /* restore state */
@@ -748,6 +819,7 @@ int capng_apply(capng_select_t set)
 			rc = -4;
 			goto try_caps;
 		}
+}
 #endif
 	}
 
@@ -774,6 +846,7 @@ try_caps:
 	// time we get here.
 	if (set & CAPNG_SELECT_AMBIENT) {
 #ifdef PR_CAP_AMBIENT
+if (HAVE_PR_CAP_AMBIENT) {
 		if (capng_have_capabilities(CAPNG_SELECT_AMBIENT) ==
 								CAPNG_NONE) {
 			if (prctl(PR_CAP_AMBIENT,
@@ -800,6 +873,7 @@ try_caps:
 			}
 		}
 		m.state = CAPNG_APPLIED;
+}
 #endif
 	}
 out:
@@ -892,7 +966,7 @@ int capng_apply_caps_fd(int fd)
 // flag to drop supp groups
 int capng_change_id(int uid, int gid, capng_flags_t flag)
 {
-	int rc, need_setgid, need_setuid;
+	int rc, ret, need_setgid, need_setuid;
 
 	// Before updating, we expect that the data is initialized to something
 	if (m.state < CAPNG_INIT)
@@ -900,11 +974,13 @@ int capng_change_id(int uid, int gid, capng_flags_t flag)
 
 	// Check the current capabilities
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 	// If newer kernel, we need setpcap to change the bounding set
 	if (capng_have_capability(CAPNG_EFFECTIVE, CAP_SETPCAP) == 0 &&
 					flag & CAPNG_CLEAR_BOUNDING)
 		capng_update(CAPNG_ADD,
 				CAPNG_EFFECTIVE|CAPNG_PERMITTED, CAP_SETPCAP);
+}
 #endif
 	if (gid == -1 || capng_have_capability(CAPNG_EFFECTIVE, CAP_SETGID))
 		need_setgid = 0;
@@ -927,8 +1003,10 @@ int capng_change_id(int uid, int gid, capng_flags_t flag)
 
 	// Change to the temp capabilities
 	rc = capng_apply(CAPNG_SELECT_CAPS);
-	if (rc < 0)
-		return -3;
+	if (rc < 0) {
+		ret = -3;
+		goto err_out;
+	}
 
 	// If we are clearing ambient, only clear since its applied at the end
 	if (flag & CAPNG_CLEAR_AMBIENT)
@@ -938,40 +1016,54 @@ int capng_change_id(int uid, int gid, capng_flags_t flag)
 	if (flag & CAPNG_CLEAR_BOUNDING) {
 		capng_clear(CAPNG_SELECT_BOUNDS);
 		rc = capng_apply(CAPNG_SELECT_BOUNDS);
-		if (rc)
-			return -8;
+		if (rc) {
+			ret = -8;
+			goto err_out;
+		}
 	}
 
 	// Change gid
 	if (gid != -1) {
 		rc = setresgid(gid, gid, gid);
-		if (rc)
-			return -4;
+		if (rc) {
+			ret = -4;
+			goto err_out;
+		}
 	}
 
 	// See if we need to init supplemental groups
 	if ((flag & CAPNG_INIT_SUPP_GRP) && uid != -1) {
 		struct passwd *pw = getpwuid(uid);
-		if (pw == NULL)
-			return -10;
+		if (pw == NULL) {
+			ret = -10;
+			goto err_out;
+		}
 		if (gid != -1) {
-			if (initgroups(pw->pw_name, gid))
-				return -5;
-		} else if (initgroups(pw->pw_name, pw->pw_gid))
-			return -5;
+			if (initgroups(pw->pw_name, gid)) {
+				ret = -5;
+				goto err_out;
+			}
+		} else if (initgroups(pw->pw_name, pw->pw_gid)) {
+			ret = -5;
+			goto err_out;
+		}
 	}
 
 	// See if we need to unload supplemental groups
 	if ((flag & CAPNG_DROP_SUPP_GRP) && gid != -1) {
-		if (setgroups(0, NULL))
-			return -5;
+		if (setgroups(0, NULL)) {
+			ret = -5;
+			goto err_out;
+		}
 	}
 
 	// Change uid
 	if (uid != -1) {
 		rc = setresuid(uid, uid, uid);
-		if (rc)
-			return -6;
+		if (rc) {
+			ret = -6;
+			goto err_out;
+		}
 	}
 
 	// Tell it we are done keeping capabilities
@@ -997,23 +1089,31 @@ int capng_change_id(int uid, int gid, capng_flags_t flag)
 	// Done
 	m.state = CAPNG_UPDATED;
 	return 0;
+
+err_out:
+	prctl(PR_SET_KEEPCAPS, 0, 0, 0, 0);
+	return ret;
 }
 
 int capng_lock(void)
 {
 	// If either fail, return -1 since something is not right
 #ifdef PR_SET_SECUREBITS
+if (HAVE_PR_SET_SECUREBITS) {
 	int rc = prctl(PR_SET_SECUREBITS,
 			1 << SECURE_NOROOT |
 			1 << SECURE_NOROOT_LOCKED |
 			1 << SECURE_NO_SETUID_FIXUP |
 			1 << SECURE_NO_SETUID_FIXUP_LOCKED, 0, 0, 0);
 #ifdef PR_SET_NO_NEW_PRIVS
+if (HAVE_PR_SET_NO_NEW_PRIVS) {
 	if (prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0))
 		return -1;
+}
 #endif
 	if (rc)
 		return -1;
+}
 #endif
 
 	return 0;
@@ -1025,8 +1125,10 @@ capng_results_t capng_have_capabilities(capng_select_t set)
 	int empty = 0, full = 0;
 
 	// First, try to init with current set
-	if (m.state < CAPNG_INIT)
-		capng_get_caps_process();
+	if (m.state < CAPNG_INIT) {
+		if (capng_get_caps_process())
+			return CAPNG_FAIL;
+	}
 
 	// If we still don't have anything, error out
 	if (m.state < CAPNG_INIT)
@@ -1061,6 +1163,7 @@ capng_results_t capng_have_capabilities(capng_select_t set)
 		}
 	}
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 	if (set & CAPNG_SELECT_BOUNDS) {
 		if (m.bounds[0] == 0)
 			empty = 1;
@@ -1075,8 +1178,11 @@ capng_results_t capng_have_capabilities(capng_select_t set)
 		else
 			return CAPNG_PARTIAL;
 	}
+} else
+	empty = 1;
 #endif
 #ifdef PR_CAP_AMBIENT
+if (HAVE_PR_CAP_AMBIENT) {
 	if (set & CAPNG_SELECT_AMBIENT) {
 		if (m.ambient[0] == 0)
 			empty = 1;
@@ -1091,6 +1197,8 @@ capng_results_t capng_have_capabilities(capng_select_t set)
 		else
 			return CAPNG_PARTIAL;
 	}
+} else
+	empty = 1;
 #endif
 	if (empty == 1 && full == 0)
 		return CAPNG_NONE;
@@ -1106,8 +1214,10 @@ capng_results_t capng_have_permitted_capabilities(void)
 	int empty = 0, full = 0;
 
 	// First, try to init with current set
-	if (m.state < CAPNG_INIT)
-		capng_get_caps_process();
+	if (m.state < CAPNG_INIT) {
+		if (capng_get_caps_process())
+			return CAPNG_FAIL;
+	}
 
 	// If we still don't have anything, error out
 	if (m.state < CAPNG_INIT)
@@ -1153,19 +1263,21 @@ static int check_inheritable(unsigned int capability, unsigned int idx)
 static int bounds_bit_check(unsigned int capability, unsigned int idx)
 {
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 	return MASK(capability) & m.bounds[idx] ? 1 : 0;
-#else
-	return 0;
+}
 #endif
+	return 0;
 }
 
 static int ambient_bit_check(unsigned int capability, unsigned int idx)
 {
 #ifdef PR_CAP_AMBIENT
+if (HAVE_PR_CAP_AMBIENT) {
 	return MASK(capability) & m.ambient[idx] ? 1 : 0;
-#else
-	return 0;
+}
 #endif
+	return 0;
 }
 
 static int v1_check(unsigned int capability, __u32 data)
@@ -1176,8 +1288,10 @@ static int v1_check(unsigned int capability, __u32 data)
 int capng_have_capability(capng_type_t which, unsigned int capability)
 {
 	// First, try to init with current set
-	if (m.state < CAPNG_INIT)
-		capng_get_caps_process();
+	if (m.state < CAPNG_INIT) {
+		if (capng_get_caps_process())
+			return 0;
+	}
 
 	// If we still don't have anything, error out
 	if (m.state < CAPNG_INIT)
@@ -1246,14 +1360,18 @@ char *capng_print_caps_numeric(capng_print_t where, capng_select_t set)
 			}
 		}
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 		if (set & CAPNG_SELECT_BOUNDS)
 			printf("Bounding Set: %08X, %08X\n",
 				m.bounds[1] & UPPER_MASK, m.bounds[0]);
+}
 #endif
 #ifdef PR_CAP_AMBIENT
+if (HAVE_PR_CAP_AMBIENT) {
 		if (set & CAPNG_SELECT_AMBIENT)
 			printf("Ambient :     %08X, %08X\n",
 				m.ambient[1] & UPPER_MASK, m.ambient[0]);
+}
 #endif
 	} else if (where == CAPNG_PRINT_BUFFER) {
 		if (set & CAPNG_SELECT_CAPS) {
@@ -1282,6 +1400,7 @@ char *capng_print_caps_numeric(capng_print_t where, capng_select_t set)
 		}
 		if (set & CAPNG_SELECT_BOUNDS) {
 #ifdef PR_CAPBSET_DROP
+if (HAVE_PR_CAPBSET_DROP) {
 			char *s;
 			// If ptr is NULL, we only room for bounding and ambient
 			if (ptr == NULL ) {
@@ -1294,10 +1413,12 @@ char *capng_print_caps_numeric(capng_print_t where, capng_select_t set)
 				s = ptr + strlen(ptr);
 			snprintf(s, 40, "Bounding Set: %08X, %08X\n",
 					m.bounds[1] & UPPER_MASK, m.bounds[0]);
+}
 #endif
 		}
 		if (set & CAPNG_SELECT_AMBIENT) {
 #ifdef PR_CAP_AMBIENT
+if (HAVE_PR_CAP_AMBIENT) {
 			char *s;
 			// If ptr is NULL, we only room for ambient
 			if (ptr == NULL ) {
@@ -1311,6 +1432,7 @@ char *capng_print_caps_numeric(capng_print_t where, capng_select_t set)
 			snprintf(s, 40, "Ambient Set: %08X, %08X\n",
 					m.ambient[1] & UPPER_MASK,
 					m.ambient[0]);
+}
 #endif
 		}
 	}
