@@ -1,5 +1,5 @@
 /* libcap-ng.h --
- * Copyright 2009,2013,2020,2022 Red Hat Inc.
+ * Copyright 2009,2013,2020-23 Red Hat Inc.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -12,9 +12,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; see the file COPYING.LIB. If not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor
+ * Boston, MA 02110-1335, USA.
  *
  * Authors:
  *      Steve Grubb <sgrubb@redhat.com>
@@ -27,11 +28,18 @@
 #include <linux/capability.h>
 #include <unistd.h>
 
+// The next 2 macros originate in sys/cdefs.h
+// gcc-analyzer notation
 #ifndef __attr_dealloc
 # define __attr_dealloc(dealloc, argno)
 # define __attr_dealloc_free
 #endif
 
+// Warn unused result
+#ifndef __wur
+# define __wur
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -57,21 +65,21 @@ typedef enum {  CAPNG_NO_FLAG=0, CAPNG_DROP_SUPP_GRP=1,
 void capng_clear(capng_select_t set);
 void capng_fill(capng_select_t set);
 void capng_setpid(int pid);
-int capng_get_caps_process(void);
+int capng_get_caps_process(void) __wur;
 int capng_update(capng_act_t action, capng_type_t type,unsigned int capability);
 int capng_updatev(capng_act_t action, capng_type_t type,
 		unsigned int capability, ...);
 
 // These functions apply the capabilities previously setup to a process
-int capng_apply(capng_select_t set);
-int capng_lock(void);
-int capng_change_id(int uid, int gid, capng_flags_t flag);
+int capng_apply(capng_select_t set) __wur;
+int capng_lock(void) __wur;
+int capng_change_id(int uid, int gid, capng_flags_t flag) __wur;
 
 // These functions are used for file based capabilities
 int capng_get_rootid(void);
 int capng_set_rootid(int rootid);
-int capng_get_caps_fd(int fd);
-int capng_apply_caps_fd(int fd);
+int capng_get_caps_fd(int fd) __wur;
+int capng_apply_caps_fd(int fd) __wur;
 
 // These functions check capability bits
 capng_results_t capng_have_capabilities(capng_select_t set);
